@@ -48,6 +48,7 @@ static void print_gss_error(OM_uint32 err_maj, OM_uint32 err_min)
 	gss_buffer_desc status_string;
 	char buf_maj[512];
 	char buf_min[512];
+        size_t status_len;
 	
 	do
 	{
@@ -59,7 +60,11 @@ static void print_gss_error(OM_uint32 err_maj, OM_uint32 err_min)
 					       &status_string);
 		if (GSS_ERROR(maj_stat))
 			break;
-		strncpy(buf_maj, (char*) status_string.value, sizeof(buf_maj));
+                status_len = status_string.length;
+                if (status_len >= sizeof(buf_maj))
+                        status_len = sizeof(buf_maj) - 1;
+		strncpy(buf_maj, (char*) status_string.value, status_len);
+                buf_maj[status_len] = '\0';
 		gss_release_buffer(&min_stat, &status_string);
 		
 		maj_stat = gss_display_status (&min_stat,
@@ -70,7 +75,11 @@ static void print_gss_error(OM_uint32 err_maj, OM_uint32 err_min)
 					       &status_string);
 		if (!GSS_ERROR(maj_stat))
 		{
-			strncpy(buf_min, (char*) status_string.value, sizeof(buf_min));
+                        status_len = status_string.length;
+                        if (status_len >= sizeof(buf_min))
+                                status_len = sizeof(buf_min) - 1;
+			strncpy(buf_min, (char*) status_string.value, status_len);
+                        buf_min[status_len] = '\0';
 			gss_release_buffer(&min_stat, &status_string);
 		}
 	} while (!GSS_ERROR(maj_stat) && msg_ctx != 0);
@@ -105,7 +114,7 @@ imap_auth_res_t imap_auth_gss (IMAP_DATA* idata, const char* method)
   /* get an IMAP service ticket for the server */
   snprintf (buf1, sizeof (buf1), "imap@%s", idata->conn->account.host);
   request_buf.value = buf1;
-  request_buf.length = strlen (buf1) + 1;
+  request_buf.length = strlen (buf1);
   maj_stat = gss_import_name (&min_stat, &request_buf, gss_nt_service_name,
     &target_name);
   if (maj_stat != GSS_S_COMPLETE)
@@ -258,7 +267,7 @@ imap_auth_res_t imap_auth_gss (IMAP_DATA* idata, const char* method)
   /* server decides if principal can log in as user */
   strncpy (buf1 + 4, idata->conn->account.user, sizeof (buf1) - 4);
   request_buf.value = buf1;
-  request_buf.length = 4 + strlen (idata->conn->account.user) + 1;
+  request_buf.length = 4 + strlen (idata->conn->account.user);
   maj_stat = gss_wrap (&min_stat, context, 0, GSS_C_QOP_DEFAULT, &request_buf,
     &cflags, &send_token);
   if (maj_stat != GSS_S_COMPLETE)
